@@ -48,6 +48,65 @@ class IngestionPipeline:
         except subprocess.CalledProcessError:
             raise RuntimeError("Working tree has tracked changes; commit or stash before indexing.")
 
+    def _drop_repo_index(self, repo_id: int, repo_name: str) -> None:
+        """Drop all indexed data for a repository (vectors and metadata).
+        
+        This clears:
+        - All chunk content and locations from metadata
+        - All vectors from LanceDB (both small and large models)
+        - All FTS5 index entries
+        
+        Args:
+            repo_id: Repository ID
+            repo_name: Repository name
+        """
+        # Delete from LanceDB (both models)
+        print(f"  Clearing vectors from LanceDB...")
+        for model in ["small", "large"]:
+            try:
+                self.lancedb.delete_repo(repo_name, model=model)
+            except Exception as e:
+                print(f"  Warning: Could not delete {model} vectors: {e}")
+        
+        # Delete from metadata database
+        print(f"  Clearing metadata...")
+        with self.metadata._connect() as conn:
+            from contextlib import closing
+            cur = conn.cursor()
+            
+            try:
+                # Get all file IDs for this repo
+                cur.execute("SELECT id FROM files WHERE repo_id = ?", (repo_id,))
+                file_ids = [row[0] for row in cur.fetchall()]
+                
+                # Delete chunk locations for these files
+                for file_id in file_ids:
+                    cur.execute("""
+                        DELETE FROM chunk_locations
+                        WHERE content_id IN (
+                            SELECT id FROM chunk_content WHERE file_id = ?
+                        )
+                    """, (file_id,))
+                
+                # Delete chunk content
+                cur.execute("DELETE FROM chunk_content WHERE repo_id = ?", (repo_id,))
+                
+                # Delete from FTS5
+                cur.execute("DELETE FROM chunks_fts WHERE repo = ?", (repo_name,))
+                
+                # Delete files
+                cur.execute("DELETE FROM files WHERE repo_id = ?", (repo_id,))
+                
+                # Delete sessions
+                cur.execute("DELETE FROM sessions WHERE repo_id = ?", (repo_id,))
+                
+                conn.commit()
+                print(f"  Metadata cleared successfully")
+            except Exception as e:
+                conn.rollback()
+                print(f"  Error clearing metadata: {e}")
+                raise
+
     def scan(self, repo_name: str, *, dry_run: bool = False, force: bool = False) -> dict:
         """Perform scanning for the named repository and persist file catalog.
 
@@ -138,11 +197,11 @@ class IngestionPipeline:
         print(f"Scan complete for {repo_name}: files_kept={result['files_kept']}, session={result['session_id']}")
 
     def index(
-        self, 
-        repo_name: str, 
-        *, 
-        dry_run: bool = False, 
-        force: bool = False, 
+        self,
+        repo_name: str,
+        *,
+        dry_run: bool = False,
+        force: bool = False,
         full_reindex: bool = False
     ) -> Dict[str, Any]:
         """Perform full indexing pipeline for the named repository.
@@ -158,7 +217,7 @@ class IngestionPipeline:
             repo_name: Name of the repository to index
             dry_run: If True, don't persist changes
             force: If True, skip clean working tree check
-            full_reindex: If True, ignore last success and process all files
+            full_reindex: If True, drop existing index and process all files
             
         Returns:
             Dictionary with session summary and counters
@@ -185,6 +244,11 @@ class IngestionPipeline:
         commit_sha = self._git(root, "rev-parse", "HEAD").strip()
         branch = self._git(root, "rev-parse", "--abbrev-ref", "HEAD").strip()
 
+        # Drop existing index if full_reindex is requested
+        if full_reindex and not dry_run:
+            print(f"Full reindex requested: dropping existing index for {repo_name}...")
+            self._drop_repo_index(repo_id, repo_name)
+
         # Get last successful commit
         last_success = self.metadata.get_last_successful_commit(repo_id)
 
@@ -236,11 +300,13 @@ class IngestionPipeline:
                     if not dry_run:
                         file_id = self.metadata.get_file_id(repo_id, path)
                         if file_id:
-                            pruned = self.metadata.prune_invalidated_content_for_file(
-                                repo_id, file_id, embed_model, current_hashes=set()
-                            )
-                            if pruned:
-                                chunks_pruned += pruned
+                            for model_name in ("small", "large"):
+                                pruned = self.metadata.prune_invalidated_content_for_file(
+                                    repo_id, file_id, model_name, current_hashes=set()
+                                )
+                                if pruned:
+                                    chunks_pruned += pruned
+                                self.lancedb.prune_file_rows(repo_name, path, model=model_name)
                     continue
                 # Skip binary files and files that don't exist
                 file_path = root / path
@@ -283,6 +349,7 @@ class IngestionPipeline:
 
                 # Build desired map
                 desired = build_desired_map(chunks)
+                desired_row_ids: set[str] = set()
 
                 # Deduplicate by text_hash
                 dedup = ChunkDeduplicator(self.metadata)
@@ -331,13 +398,13 @@ class IngestionPipeline:
                     payload = []
                     fts_chunks = []  # For FTS5 indexing
                     for h, occs in desired.items():
+                        content_id = mapping.get(h)
                         vec = hash_to_vec.get(h)
-                        if vec is None:
-                            continue  # unchanged hash
-                        for occ in occs:
+                        for idx, occ in enumerate(occs):
                             row_id = f"{repo_id}:{file_id}:{embed_model}:{h}:{occ['start_line']}:{occ['end_line']}"
-                            content_id = mapping.get(h)
-                            
+                            desired_row_ids.add(row_id)
+                            if vec is None:
+                                continue  # unchanged hash
                             payload.append({
                                 'id': row_id,
                                 'vector': vec,
@@ -361,7 +428,7 @@ class IngestionPipeline:
                             })
                             
                             # Prepare chunk for FTS5 indexing (only for first occurrence per hash)
-                            if content_id and occ == occs[0]:  # First occurrence only
+                            if content_id and idx == 0:  # First occurrence only
                                 # Find the chunk text for this hash
                                 chunk_text = None
                                 for chunk in chunks:
@@ -386,6 +453,12 @@ class IngestionPipeline:
                     if fts_chunks and not dry_run:
                         self.metadata.bulk_index_chunks_for_fts(fts_chunks)
 
+                    # Prune any stale vectors for this file/model
+                    if desired_row_ids:
+                        self.lancedb.prune_file_rows(repo_name, path, model=embed_model, keep_ids=desired_row_ids)
+                    else:
+                        self.lancedb.prune_file_rows(repo_name, path, model=embed_model)
+
                 # Update counters
                 files_done += 1
                 chunks_indexed += len(new_hashes)
@@ -405,17 +478,43 @@ class IngestionPipeline:
             try:
                 file_id = self.metadata.get_file_id(repo_id, path)
                 if file_id:
-                    pruned_count = self.metadata.prune_invalidated_content_for_file(
-                        repo_id, file_id, embed_model, current_hashes=set()
-                    )
+                    total_pruned = 0
+                    for model_name in ("small", "large"):
+                        pruned_count = self.metadata.prune_invalidated_content_for_file(
+                            repo_id, file_id, embed_model=model_name, current_hashes=set()
+                        )
+                        if pruned_count:
+                            total_pruned += pruned_count
+                        self.lancedb.prune_file_rows(repo_name, path, model=model_name)
                     files_done += 1
-                    chunks_pruned += pruned_count
-                    print(f"  {path}: deleted, {pruned_count} chunks pruned")
+                    chunks_pruned += total_pruned
+                    print(f"  {path}: deleted, {total_pruned} chunks pruned")
             except Exception as e:
                 error_logger.log_file_error(f"deleted: {path}", e)
                 print(f"Error processing deleted file {path}: {e}")
                 continue
 
+        # Prune any ignored files that were previously indexed
+        # This handles files that were committed before being added to .gitignore
+        if not dry_run:
+            print(f"\nPruning previously-indexed ignored files for {repo_name}...")
+            all_files = self.metadata.get_all_files_for_repo(repo_id)
+            for file_record in all_files:
+                file_path = file_record["path"]
+                file_id = file_record["id"]
+                
+                # Check if file matches ignore patterns
+                if ignore_spec.match_file(file_path):
+                    # Prune all content for this file across all embedding models
+                    for model in ["small", "large"]:
+                        pruned_count = self.metadata.prune_invalidated_content_for_file(
+                            repo_id, file_id, embed_model=model, current_hashes=set()
+                        )
+                        if pruned_count > 0:
+                            chunks_pruned += pruned_count
+                            print(f"  {file_path}: pruned {pruned_count} ignored chunks (model={model})")
+                        self.lancedb.prune_file_rows(repo_name, file_path, model=model)
+        
         # Update session counters
         if not dry_run:
             self.metadata.bump_session_counters(
